#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

#=============================================================================
# RNA メインオブジェクト
#=============================================================================
package RNA;

use strict;
use LWP::UserAgent;
# use XML::RSS;
use RNA::Utils qw( dc_date );
use RNA::RSS;
use RNA::AntennaTree;
use RNA::Hatena;
use RNA::Trackback;
use Jcode;
use CGI;
my $cgi = new CGI;


use vars qw($VERSION);

$VERSION = '1.2 (build 0724)';


sub new {
    my $class = shift;
	my $self = {};
    
    bless $self, $class;
    
    $self->initialize(@_);
    
    return $self;
}

sub initialize {
    my $self = shift;
    my %hash = @_;

    # configuration
    if($hash{Config}){
		require $hash{Config};
	    $self->{Config} = RNA::Config->new;
	}
	else {
		die "Content-type text/plain\n\nConfig File not Found\n";
	}

	$self->{VERSION} = $RNA::VERSION;
	$self->{Config}->{Template}->{Element}->{RNAVersion} = $self->{VERSION};
	
	$self->{Date} = '';

	$self->{MainIndex} = '';
	
	$self->{SiteList} = {
		RSS      => {},
		RSSArray => []
	};

	$self->{RecentSiteList} = {
		RSS      => {},
		AntennaTree => {}
	};

	$self->{RecentEntryList} = {
		RSS      => {},
		AntennaTree => {}
	};

	$self->{RecentSiteEntryList} = {
		RSS      => {},
		AntennaTree => {}
	};

	# Trackback の情報を格納
	# $self->{Trackback}->{"URI"} = $antenna_tree;
	$self->{Trackback} = {};

	#-------------------------------------------
	# 現在時刻の設定（Dublin Core 形式）
	#-------------------------------------------
	my ($s,$m,$h,$md,$M,$y,$wd,$yd,$isdst) = localtime(time);
	my $month = ($M + 1);
	$y += 1900;
	$self->{Date} = main::dc_date($y,$month,$md,$h,$m,$s) . '+09:00';
}


# $rna->run(Type=>'Update', TransferURI=>$transfer_uri, Response => $redirect);
sub run
{
    my $self = shift;
    my $config = $self->{Config};
	my %hash = @_;

	my $transfer_uri = $hash{TransferURI};
	my $response = $hash{Response};
	my $type = $hash{Type};

	my $header = $cgi->header(-type=>'text/html', 
		-charset => $config->{Encoding});

	if($config->{Update}->{Background}) {
	# バックグラウンド処理
		if($type eq 'Update') {
			$response = qq($header
<html>
	<head>
		<title>Refreshing RNA...</title>
		<meta http-equiv="Refresh" content="3;URL=$transfer_uri">
	</head>
	<body bgcolor="#eeeeee" text="#bbbbbb">
		<h2>Refreshing RNA...</h2><br>
		Please wait for 3 second to back, or click [<a href="$transfer_uri">Back</a>].<br>
		It might take a while to check all sites you register.
	</body>
</html>
);
		}

		my $pid = '';
		$|=1; # 即時出力
		if($pid = fork) {
			# 親プロセス （ブラウザへのレスポンス）
			print $response;
			close(STDOUT);
		}
		elsif (defined $pid) {
			# 子プロセス （更新処理）
			close(STDOUT);
			$self->build;
			$self->print;
			exit;
		}
		else {
		# エラー
			&die_handler("プロセスを生成できません．\n");
		}	
	}
	else {
	# 通常の処理
		$self->build;
		$self->print;
		print $response;
	}
}

sub build 
{
    my $self = shift;
    my $config = $self->{Config};

	#---------------------------------------------------
	# リストの作成
	#---------------------------------------------------
    # 設定ファイルから，RSS のリストの取り出し
	my $list_rss = new RNA::RSS;
	$list_rss->parsefile($config->{SiteList}->{File});
	$self->{SiteList}->{RSS} = $list_rss;

	$self->set_rss_array;

	#---------------------------------------------------
	# Webページの作成
	#---------------------------------------------------
	# トップページの作成
	$self->make_main_index;

	# dc:date 順にソートしたサイトのリスト
	$self->make_site_list;

	# 最新記事リストの作成
	$self->make_recent_entry_list;

	# サイトごとの記事リストの作成
	$self->make_recent_site_entry_list;
}


sub print 
{
	my $self = shift;
	my $config = $self->{Config};


	#----------------------------------------
	# Main Index ページ
	#----------------------------------------
	my $str = $self->{MainIndex};
	main::print_file($config->{MainIndex}->{File}, \$str);

	#----------------------------------------
	# Recent Site リスト
	#----------------------------------------
	{
		my $str = $self->{RecentSiteList}->{RSS}->as_string;
		main::print_file($config->{RecentSiteList}->{RSS}, \$str);

		my $antenna = $self->{RecentSiteList}->{AntennaTree};

		my $template = main::read_file($config->{Template}->{RecentSiteList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		main::print_file($config->{RecentSiteList}->{HTML}, $str_ref);
	}

	#----------------------------------------
	# Recent Entry リスト
	#----------------------------------------
	{
		my $str = $self->{RecentEntryList}->{RSS}->as_string;
		main::print_file($config->{RecentEntryList}->{RSS}, \$str);

		my $antenna = $self->{RecentEntryList}->{AntennaTree};
		my $template = main::read_file($config->{Template}->{RecentEntryList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		main::print_file($config->{RecentEntryList}->{HTML}, $str_ref);
	}

	#----------------------------------------
	# Recent Site Entry リスト
	#----------------------------------------
	{
		my $str = $self->{RecentSiteEntryList}->{RSS}->as_string;
		main::print_file($config->{RecentSiteEntryList}->{RSS}, \$str);

		my $antenna = $self->{RecentSiteEntryList}->{AntennaTree};
		my $template 
			= main::read_file($config->{Template}->{RecentSiteEntryList});
		my $str_ref = $antenna->apply_template($template, 
			$config->{Template}->{Element});

		main::print_file($config->{RecentSiteEntryList}->{HTML}, $str_ref);
	}
}


# $self->set_rss_array;
#
# 最近更新されたページ順に，RSS を配列に入れ，そのリファレンスを返す．
sub set_rss_array
{
	my $self = shift;
	my $list_rss = $self->{SiteList}->{RSS};
	my $config = $self->{Config};
	my $empty_content = $config->{SiteList}->{EmptyContent};
	my $hatena_flag = $config->{HatenaFlag};

	my @rss_array = ();
	foreach my $site (@{$list_rss->{items}}){
		my $uri = $site->{link};
		my $subject = $site->{dc}->{subject};

		# RSS ドキュメント取得
		my $str = main::get_html($uri);
#		Jcode::convert(\$str, 'utf8', Jcode::getcode(\$str));	# UTF-8 に変換
		Jcode::convert(\$str, 'utf8');	# UTF-8 に変換

		# はてなのページを パース
		if($subject eq $hatena_flag) {
			my $hatena = new RNA::Hatena(URI => $uri);
			$hatena->parse_html(\$str);
			push (@rss_array, @{$hatena->{RSSArray}});
			next;
		}
		
		# parse
		my $rss = new RNA::RSS;
		$rss->parse($str);

		#---------------------------------------------------------
		# $site->{description}, $site->{title}, $site->{dc}->{***}
		# がある場合（設定ファイルにある場合）は，それを優先
		#---------------------------------------------------------
		foreach my $el ('title', 'description'){
			if($site->{$el} && $site->{$el} ne $empty_content)
			{
				$rss->{channel}->{$el} = $site->{$el};
			}
		}
		# Dublin Core モジュールについて
		foreach my $el ('subject', 'creator') {
			if($site->{dc}->{$el} 
				&& $site->{dc}->{$el} ne $empty_content){
				$rss->{channel}->{dc}->{$el} = $site->{dc}->{$el};
			}
		}

		# dc:source に，title をセット
		if($site->{title} && $site->{title} ne $empty_content)	{
			foreach my $new_item (@{$rss->{items}}) {
				$new_item->{dc}->{source} = $site->{title};
			}
		}

		#---------------------------------------------------------
		# 日付で，<item> をソート
		#---------------------------------------------------------
		my @a = sort 
			{ main::comp_dc_date($a->{'dc'}->{'date'}, $b->{'dc'}->{'date'})} 
			@{$rss->{items}};
		$rss->{items} = \@a;

		push (@rss_array, $rss);
	}

	# channel/dc:date の降順にソート
	my @ret = sort
		{main::comp_dc_date(
			$a->{channel}->{dc}->{date}, $b->{channel}->{dc}->{date})
		} 
		@rss_array;

	$self->{SiteList}->{RSSArray} = \@ret;

	# Trackback 情報の収集
	#if($config->{TrackBackTrace}){
	#	$self->set_trackbacks;
	#}
}


# $self->set_trackbacks;
#
# 前提  : $self->{SiteList}->{RSSArray} がセットされていること
#sub set_trackbacks
#{
#	my $self = shift;
#	my @rss_array = @{$self->{SiteList}->{RSSArray}};
#
#	my @items = ();
#	foreach my $rss (@rss_array) {
#		push @items, @{$rss->{items}};
#	}
#
#	foreach my $item (@items) {
#		my $tree = RNA::Trackback::get_trackback_tree($item->{link});
#		if($tree) {
#			$self->{Trackback}->{$item->{link}} = $tree;
#		}
#	}
#}

# トップページの作成
# $self->make_main_index;
sub make_main_index
{
	my $self = shift;
	my $config = $self->{Config};
	
	my $template = main::read_file($config->{Template}->{MainIndex});

	my %text_hash = %{$config->{Template}->{Element}};
	$text_hash{RNAPageTitle}       = $config->{MainIndex}->{Title};
	$text_hash{RNAPageDescription} = $config->{MainIndex}->{Description};
	$text_hash{RNAPageDate}        = $self->{Date};

	$text_hash{RNASiteList}        = $config->{SiteList}->{File};
	
	$text_hash{RNARecentSiteRSS}   = $config->{RecentSiteList}->{RSS};
	$text_hash{RNARecentSiteHTML}  = $config->{RecentSiteList}->{HTML};
	$text_hash{RNARecentSiteXML}   = $config->{RecentSiteList}->{XML};

	$text_hash{RNARecentEntryRSS}  = $config->{RecentEntryList}->{RSS};
	$text_hash{RNARecentEntryHTML} = $config->{RecentEntryList}->{HTML};
	$text_hash{RNARecentEntryXML}  = $config->{RecentEntryList}->{XML};

	$text_hash{RNARecentSiteEntryRSS}  = $config->{RecentSiteEntryList}->{RSS};
	$text_hash{RNARecentSiteEntryHTML} = $config->{RecentSiteEntryList}->{HTML};
	
	foreach my $t (keys %text_hash){
		$template =~ s/\<\$$t\$\>/$text_hash{$t}/ges;
	}

	$self->{MainIndex} = $template;
}

# dc:date 順にソートしたサイトのリスト
# $self->make_site_list;
sub make_site_list
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	#--------------------------------------
	# RSS 作成
	#--------------------------------------
	my $rss = new RNA::RSS;
	&main::set_rss($rss, 
		$config->{RecentSiteList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteList}->{RSS}, 
		$config->{RecentSiteList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});

	foreach my $r (@{$rss_array_ref}){
		$rss->add_item(%{$r->{channel}});
	}

	$self->{RecentSiteList}->{RSS} = $rss;


	#--------------------------------------
	# テンプレートからのHTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentSiteList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteList}->{HTML}, 
		$config->{RecentSiteList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_rss_array($rss_array_ref);

	$self->{RecentSiteList}->{AntennaTree} = $antenna;
}


# 最新記事リストの作成
# $self->make_recent_entry_list;
sub make_recent_entry_list 
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	my @items = ();
	foreach my $a (@{$rss_array_ref}){
		push (@items, @{$a->{items}} )
	}

	# 日付でソート
	my @a = sort 
		{ main::comp_dc_date($a->{'dc'}->{'date'}, $b->{'dc'}->{'date'})} 
		@{items};
	splice @a, $config->{RecentEntryList}->{Number};
	my $items_ref = \@a;

	#---------------------------------------------------
	# RSS の作成
	#---------------------------------------------------
	my $rss = new RNA::RSS;
	main::set_rss($rss, 
		$config->{RecentEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentEntryList}->{RSS}, 
		$config->{RecentEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	foreach my $item (@{$items_ref}){
		$rss->add_item(%{$item});
	}

	$self->{RecentEntryList}->{RSS} = $rss;

	#--------------------------------------
	# テンプレートからのHTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentEntryList}->{HTML}, 
		$config->{RecentEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_items_array($items_ref);

	$self->{RecentEntryList}->{AntennaTree} = $antenna;
}


# サイトごとの記事リストの作成
# $self->make_recent_site_entry_list;
sub make_recent_site_entry_list
{
	my $self = shift;
	my $config = $self->{Config};
	my $rss_array_ref = $self->{SiteList}->{RSSArray};

	#--------------------------------------
	# HTML作成
	#--------------------------------------
	my $antenna = new RNA::AntennaTree;
	$antenna->initialize($config->{RecentSiteEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteEntryList}->{HTML},
		$config->{RecentSiteEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});
	$antenna->set_from_rss_array($rss_array_ref);

	foreach my $site (@{$antenna->{items}}){
		my $configured_num = $config->{RecentSiteEntryList}->{NumberPerSite};
		my $num_per_site = 
			$configured_num < scalar(@{$site->{items}}) ? 
				$configured_num : scalar(@{$site->{items}});
		splice @{$site->{items}}, $num_per_site;
	}

	$self->{RecentSiteEntryList}->{AntennaTree} = $antenna;

	#---------------------------------------
	# RSS 
	#---------------------------------------
	my $rss = new RNA::RSS;
	&main::set_rss($rss, $config->{RecentSiteEntryList}->{Title}, 
		$config->{BaseURI} . $config->{RecentSiteEntryList}->{RSS}, 
		$config->{RecentSiteEntryList}->{Description}, 
		$self->{Date}, 
		$config->{Owner}, $config->{Language});

	my @all_items = ();
	foreach my $r (@{$rss_array_ref}){
		my $configured_num = $config->{RecentSiteEntryList}->{NumberPerSite};
		my $num_per_site = 
			$configured_num < scalar(@{$r->{items}}) ? 
			$configured_num : scalar(@{$r->{items}});
		my @contents = @{$r->{items}};
		splice @contents, $num_per_site;
		push(@all_items, @contents);
	}

	$rss->{items} = \@all_items;

	$self->{RecentSiteEntryList}->{RSS} = $rss;
}





1;
__END__

